/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.functionality;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.vocabulary.*;
import java.io.*;
import java.util.*;
/**
   Implementation of {@link RDFFile} using Jena.
   <p>
   This requires the <a href="http://jena.sourceforge.net/">Jena API</a>
   to be installed on your system and listed in java's CLASSPATH.

   @author James Nash
*/
public class JenaFile implements RDFFile{

    // the RDF file that is loaded. null if nothing si loaded.
    private File file;


    // the Jena model of the loaded RDF data
    private Model model;



    /**
       Constructs a new JenaFile object that is capable of reading,
       modifying and writing RDF/XML data. 
    */
    public JenaFile(){

        model = null;
	file = null;

    }


    /**
       Creates blank RDF data in memory.
       If previous RDF data is present it is cleared.
    */
    public void newFile(){

	// clear old file
	file = null;
	// make blank model
        model = ModelFactory.createDefaultModel();

	// teach Jena some extra namespaces...
	model.setNsPrefix("foaf","http://xmlns.com/foaf/0.1/");
	model.setNsPrefix("cc","http://web.resource.org/cc/");
	model.setNsPrefix("dctype","http://purl.org/dc/dcmitype/");
        
    }


    /**
       Reads in RDF data from a given RDF/XML file.

       @param file the file to load
       @throws RdfIOException if the file cannot be loaded or contains invalid RDF/XML
    */
    public void loadFile( File file ) throws RdfIOException{
	this.file = file;
        // create new model        
        model = ModelFactory.createDefaultModel();

	// teach Jena some extra namespaces...
	model.setNsPrefix("foaf","http://xmlns.com/foaf/0.1/");
	model.setNsPrefix("cc","http://web.resource.org/cc/");
	model.setNsPrefix("dctype","http://purl.org/dc/dcmitype/");

	// create an RDF reader
        RDFReader reader = model.getReader();

	// set some settings for the reader?
	reader.setProperty("WARN_RESOLVING_URI_AGAINST_EMPTY_BASE","EM_IGNORE");

        try{
            
            InputStream in = new FileInputStream( file );
        
            // read file into Jena model
            reader.read(model,in,"");
            in.close();

        }
        catch( Exception e ){
            // deal with file not found error
            if ( e instanceof FileNotFoundException ){
            
                System.err.println( "File read error: "+e.getMessage() );
            
                throw new RdfIOException( RdfIOException.FILE_NOT_FOUND );
            
            }
            // deal with rdf reading errors
            else if( e instanceof com.hp.hpl.jena.shared.JenaException ){
            
                // a JenaException at this point indicates that there was
                // some problem reading the file            
                System.err.println( "RDF read error: "+e.getMessage() );
            
                throw new RdfIOException( RdfIOException.INVALID_RDF );
            
            }
            else{
                
                System.err.println( "Unknown read error: "+e.getMessage() );
                
                throw new RdfIOException( RdfIOException.OTHER_READ_ERROR );
            }
            
        }
    }


    /**
       Saves current RDF data to the current file.

       @throws RdfIOException if there is a write error
    */
    public void saveFile() throws RdfIOException{
        // just write model out to STDOUT for testing
        
	RDFWriter writer = model.getWriter();
	// need this to allow relatve URLs...
	writer.setProperty("allowBadURIs","true");
	writer.setProperty("showXmlDeclaration","true");


	try{

	    OutputStream out = new FileOutputStream( file );
	    
	    writer.write(model, out, "");
	    out.close();
	}
	catch( Exception e ){

	     // deal with file not found error
            if ( e instanceof FileNotFoundException ){
            
                System.err.println( "File write error: "+e.getMessage() );
		
                throw new RdfIOException( RdfIOException.NO_WRITE_PERM );
            
            }


	}
	
    }



    /**
       Save loaded RDF data to a file different from the current one.

       @param file the file to write to
       @throws RdfIOException if there is a write error
    */
    public void saveFile( File file ) throws RdfIOException{
	this.file = file;
        
        saveFile();	
    }



    /**
       Returns the name of the file from which the currently loaded RDF data originates. This is usually the name of the file which was loaded unless a new file has been created since then or the data was written to a different file.

       @return the current file name (null if there is no filename).
       @see #newFile()
       @see #loadFile(File)
       @see #saveFile(File)
     */
    public String getFileName(){
	String output = null;

	if ( file != null ){
	    output = file.getName();
	}
	return output;

    }


    //================== Methods for working with RDF ============================
    
    /**
       Retrieves all resources from the currently loaded data that have
       properties describing them.
       This includes anonymous resources.
       
       @return array of all the resources that are currently loaded
    */
    public RDFResource[] getResources(){
    
        // get resources from jena model
        ResIterator resIt = model.listSubjects();
    
        // vector to build list
        Vector tmp = new Vector();
        Resource tmpR;
        // put resources into vector...
        while ( resIt.hasNext() ){
            tmpR = resIt.nextResource();
	    	    
	    // check if resource is actually a container
	    if( tmpR.hasProperty( RDF.type, RDF.Bag ) || tmpR.hasProperty( RDF.type, RDF.Seq ) || tmpR.hasProperty( RDF.type, RDF.Alt ) ){		
		//System.out.println("Resource not added coz its a container!");
	    }
	    else{

		tmp.add( tmpR );
	    }

        }
        // prepare output array
        RDFResource[] res = new JenaResource[ tmp.size() ];
        
        // transfer resources to RDFResource array
        for (int i=0; i<res.length; i++){
	    
	    res[i] = new JenaResource( (Resource) tmp.elementAt(i) );
        }
        
    
        // return array
        return res;
    }

    

    /**
       Create a new Resource.
       If passed null instead of a URI an anonymous resource will
       be created.

       @param uri the URI of what the resource is describing or null
       @return a new RDFResource
    */
    public RDFResource createResource(String uri){

	
	// create the new resource
	Resource myres;

	if ( uri == null ){
	    //anonymous
	    myres = model.createResource();
	}
	else{
	    myres = model.createResource( uri );
	}

	JenaResource output = new JenaResource( myres );	

	return output;
    }

    /**
       Creates a new anonymous Resource.
      
       @return a new anonymous RDFResource
    */
    public RDFResource createResource(){
	return createResource( null );
    }




    /**
       Creates a new Bag container.

       @param uri the URI of what the bag is describing
       @return a new RDFBag
    */
    public RDFBag createBag(String uri){
	Bag bag = model.createBag( uri );
	RDFBag output = new JenaBag( bag );
	return output;
    }


    /**
       Creates a new anonymous bag container.
      
       @return a new anonymous RDFBag
    */
    public RDFBag createBag(){
	Bag bag = model.createBag();
	RDFBag output = new JenaBag( bag );
	return output;
    }


    /**
       Creates a new Seq container.

       @param uri the URI of what the seq is describing
       @return a new RDFSeq
    */
    public RDFSeq createSeq(String uri){
	Seq seq = model.createSeq( uri );
	RDFSeq output = new JenaSeq( seq );
	return output;
    }

    /**
       Creates a new anonymous seq container.
      
       @return a new anonymous RDFSeq
    */
    public RDFSeq createSeq(){
	Seq seq = model.createSeq();
	RDFSeq output = new JenaSeq( seq );
	return output;
    }

    /**
       Creates a new Alt container.

       @param uri the URI of what the alt is describing
       @return a new RDFAlt
    */
    public RDFAlt createAlt(String uri){
	Alt alt = model.createAlt( uri );
	RDFAlt output = new JenaAlt( alt );
	return output;
    }

    /**
       Creates a new anonymous alt container.
      
       @return a new anonymous RDFAlt
    */
    public RDFAlt createAlt(){
	Alt alt = model.createAlt();
	RDFAlt output = new JenaAlt( alt );
	return output;
    }



    /**
       Creates a new <code>RDFPredicate</code> for when building a statement.
       
       @param type the Property's type (usually a URI)
       @return a new RDFPredicate
    */
    public RDFPredicate createPredicate(String type){

	Property prop = model.createProperty( type );
	JenaPredicate output = new JenaPredicate(prop);
	return output;
    }

    /**
       Create a literal <code>RDFObject</code> for when building a statement.

       @param val the object's literal value
       @return a new RDFObject
    */
    public RDFObject createObject(String val){
	JenaObject output = new JenaObject( val );
	return output;
    }

    /**
       Create a Resource <code>RDFObject</code> for when building a statement.

       @param res the RDFResource to use
       @return a new RDFObject
    */
    public RDFObject createObject(RDFResource res){
	JenaResource jres = (JenaResource) res;
	JenaObject output = new JenaObject( jres.getJenaRes() );

	return output;
    }


    /**
       Create an <code>RDFObject</code> that is a container.

       @param con the RDFContainer to use
       @return a new RDFObject
    */
    public RDFObject createObject(RDFContainer con){
	
	JenaContainerAdaptor tmp = null;

	// there are 3 kinds of container...
	if ( con instanceof RDFBag ){
	    tmp = (JenaBag) con;
	}
	else if ( con instanceof RDFSeq ){
	    tmp = (JenaSeq) con;
	}
	else if ( con instanceof RDFAlt ){
	    tmp = (JenaAlt) con;
	}

	RDFObject output = new JenaObject( tmp.getJenaNode() );

	return output;
    }



    /**
       Shortens the URIs of properties.
       First it checks with a set of predefined descriptions
       of common properties. If no match is found the
       last part of the property's URI is chopped off and
       returned.

       @param uri the property uri to shorten
       @return a short description
    */
    public static String shorten( String uri ){

	// create temp model        
        Model tmpModel = ModelFactory.createDefaultModel();

	// teach Jena some extra namespaces...
	tmpModel.setNsPrefixes(com.hp.hpl.jena.shared.PrefixMapping.Standard);
	tmpModel.setNsPrefix("foaf","http://xmlns.com/foaf/0.1/");
	tmpModel.setNsPrefix("cc","http://web.resource.org/cc/");
	tmpModel.setNsPrefix("dctype","http://purl.org/dc/dcmitype/");

	// let Jena try to shorten it for us...
	String output = tmpModel.usePrefix( uri );

	// if it didn't work:
	if ( output == uri ){

	    // trim what is probably the namespace off the URI
	    String ext = uri;	
	    int i = uri.lastIndexOf('#');
	    if (i == -1 ){
		i = uri.lastIndexOf('/');
	    }
	    
	    if (i > 0 &&  i < uri.length() - 1) {
		ext = uri.substring(i+1);
	    }
	
	    output = ext;
	}


	// return result
        return output;

    }


    // known resource types
    private String[] types = {
	"http://web.resource.org/cc/Agent",
	"http://web.resource.org/cc/License",
	"http://web.resource.org/cc/Work",

	"http://purl.org/dc/dcmitype/Event",
	"http://purl.org/dc/dcmitype/PhysicalObject",
	"http://purl.org/dc/dcmitype/Software",
	"http://purl.org/dc/dcmitype/Sound",
	"http://purl.org/dc/dcmitype/Text",

	"http://xmlns.com/foaf/0.1/Document",
	"http://xmlns.com/foaf/0.1/Group",
	"http://xmlns.com/foaf/0.1/Image",
	"http://xmlns.com/foaf/0.1/OnlineAccount",
	"http://xmlns.com/foaf/0.1/Organization",
	"http://xmlns.com/foaf/0.1/Person",
	"http://xmlns.com/foaf/0.1/Project",

	"http://purl.org/rss/1.0/channel",
	"http://purl.org/rss/1.0/item"
    };

    /**
       Returns a list of known resource types.
       The list is an array of 2 element arrays.
       Each sub-array has a discriptive name as
       its first element and the full URI of the
       tpye as its second element.

       @return list of know resource type names and URIs
     */
    public String[][] getResTypes(){
	
	String[][] output = new String[ types.length ][2];

	for ( int i=0; i<types.length; i++){

	    output[i][0] = shorten( types[i] );
	    output[i][1] = types[i];
		
	}

	return output;

    }


    // known property types...
    private Object[] props = {
	"http://web.resource.org/cc/license",
	"http://web.resource.org/cc/permits",
	"http://web.resource.org/cc/requires",
	"http://web.resource.org/cc/prohibits",
	"http://web.resource.org/cc/derivativeWork",

   	DC.contributor,
	DC.coverage,
	DC.creator,
	DC.date,
	DC.description,
	DC.format,
	DC.language,
	DC.publisher,
	DC.relation,
	DC.rights,
	DC.source,
	DC.subject,
	DC.title,
	DC.type,

	"http://xmlns.com/foaf/0.1/accountName",
	"http://xmlns.com/foaf/0.1/accountServiceHomepage",
	"http://xmlns.com/foaf/0.1/aimChatID",
	"http://xmlns.com/foaf/0.1/currentProject",
	"http://xmlns.com/foaf/0.1/depiction",
	"http://xmlns.com/foaf/0.1/depicts",
	"http://xmlns.com/foaf/0.1/firstName",
	"http://xmlns.com/foaf/0.1/holdsAccount",
	"http://xmlns.com/foaf/0.1/homepage",
	"http://xmlns.com/foaf/0.1/icqChatID",
	"http://xmlns.com/foaf/0.1/img",
	"http://xmlns.com/foaf/0.1/jabberID",
	"http://xmlns.com/foaf/0.1/knows",
	"http://xmlns.com/foaf/0.1/member",
	"http://xmlns.com/foaf/0.1/mbox",
	"http://xmlns.com/foaf/0.1/mbox_sha1sum",
	"http://xmlns.com/foaf/0.1/msnChatID",
	"http://xmlns.com/foaf/0.1/name",
	"http://xmlns.com/foaf/0.1/nick",
	"http://xmlns.com/foaf/0.1/pastProject",
	"http://xmlns.com/foaf/0.1/schoolHomepage",
	"http://xmlns.com/foaf/0.1/surname",
	"http://xmlns.com/foaf/0.1/title",
	"http://xmlns.com/foaf/0.1/weblog",
	"http://xmlns.com/foaf/0.1/workplaceHomepage",
	"http://xmlns.com/foaf/0.1/workInfoHomepage",

	RDFS.seeAlso,

	RSS.description,
	RSS.image,
	RSS.items,
	RSS.link,
	RSS.name,
	RSS.title,
	RSS.url
    };


    /**
       Returns a list of known property types.

       @return list of known predicates
    */
    public RDFPredicate[] getPropTypes(){

	RDFPredicate[] output = new RDFPredicate[ props.length ];

	for ( int i=0; i<props.length; i++){
	    // fill in the JenaProperties
	    if ( props[i] instanceof String ){
		output[i] = createPredicate( ((String) props[i]) );
	    }
	    else{
		output[i] = new JenaPredicate( ((Property) props[i]) );
	    }
	}

	return output;
    }


}
