/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.functionality;

import com.hp.hpl.jena.rdf.model.*;
/**
   Implementation of {@link RDFObject} using Jena.
   <p>
   This requires the <a href="http://jena.sourceforge.net/">Jena API</a>
   to be installed on your system and listed in java's CLASSPATH.

   @author James Nash
*/
public class JenaObject implements RDFObject{


    /**
       The type code of this object.
    */
    private int type;

    /**
       The literal value of this object.
       (May be null if this is not a literal object)
    */
    private String lit;

    /**
       The Jena RDFNode value of this object.
       (May be null if this is a literal object)
    */
    private RDFNode node;



    /**
       Constructs a new literal JenaObject.

       @param lit the value of this object
    */
    public JenaObject( String lit ){
	this.lit = lit;
	

	// need to set node to an RDF Literal
	// for compatibility with other Jena classes that
	// may call getJenaNode()
	Model tmp = ModelFactory.createDefaultModel();
	this.node = tmp.createLiteral( lit, false);

	type = RDFObject.LITERAL;
    }

    /**
       Constructs a new non-literal JenaObject.
       The type of the supplied node is determined
       automatically.
    */
    public JenaObject( RDFNode node ){
	this.lit = null;
	this.node = node;

	// containers are subclasses of resources, so
	// check for them first!
	if ( node instanceof Container ){
	    // OK, now we know it's a container
	    // figure out which kind!
	    if ( node instanceof Bag ){
		// it's a bag
		type = RDFObject.BAG;
		//System.err.println("Object is Bag!");
	    }
	    else if ( node instanceof Seq ){
		// it's a seq
		type = RDFObject.SEQ;
		//System.err.println("Object is Sequence!");
	    }
	    else if ( node instanceof Alt ){
		// it's a seq
		type = RDFObject.ALT;
		//System.err.println("Object is Alternatives!");
	    }
	    else{
		// wtf???
		type = RDFObject.UNKNOWN;
	    }
	}
	else if ( node instanceof Resource ){
	    type = RDFObject.RESOURCE;

	    //Resource tempy = (Resource) node;
	    //System.out.println("Object with: "+tempy.getURI());

	}
	else{
	    type = RDFObject.UNKNOWN;
	}

    }

    /**
       Query the type of this RDF object.
       The returned value can be compared to the
       constants defined in {@link RDFObject} to determine
       of what type the object is.

       @see RDFObject#LITERAL
       @see RDFObject#RESOURCE
       @see RDFObject#BAG
       @see RDFObject#SEQ
       @see RDFObject#ALT
       @see RDFObject#UNKNOWN
       @return object type code
    */
    public int getObjType(){
	return type;
    }


    /**
       Retrieve the literal value.
       If this object is of type literal then the value
       will be returned. Otherwise null will be returned.

       @return value or null
    */
    public String getLiteral(){
	return lit;
    }


    /**
       Retrieve Resource.
       If this object is of type resource then the
       resource will be returned. Otherwise null will be returned.

       @return resource or null
    */
    public RDFResource getResource(){

	RDFResource output = null;

	if ( getObjType() == RESOURCE ){
	    output = new JenaResource( (Resource) node );
	}

	return output;
    }


    /**
       Retrieve bag.
       If this object is of type bag then
       that container will be returned. Otherwise null
       will be returned.

       @return bag or null
    */
    public RDFBag getBag(){

	RDFBag output = null;

	if ( getObjType() == BAG ){
	    output = new JenaBag( (Bag) node );
	}

	return output;
    }

    /**
       Retrieve sequence.
       If this object is of type seq then
       that container will be returned. Otherwise null
       will be returned.

       @return sequence or null
    */
    public RDFSeq getSeq(){

	RDFSeq output = null;

	if ( getObjType() == SEQ ){
	    output = new JenaSeq( (Seq) node );
	}

	return output;
    }

    /**
       Retrieve alternatives.
       If this object is of type alt then
       that container will be returned. Otherwise null
       will be returned.

       @return alternatives or null
    */
    public RDFAlt getAlt(){

	RDFAlt output = null;

	if ( getObjType() == ALT ){
	    output = new JenaAlt( (Alt) node );
	}

	return output;
    }



    /**
       Get Jena RDFNode of resource or container.
       Intended for use by the other Jena* classes.

       @return the resource or container as a Jena RDFNode
    */
    public RDFNode getJenaNode(){
	return node;
    }


    /**
       Return a unique ID for this object.
       Use this to compare to objects.

       @return unique id string for this object
    */
    public String getID(){
	String output = getObjType()+":";
	if ( getObjType() == LITERAL ){
	    output += lit;
	}
	else{
	    Resource res =  (Resource) node;
	    if ( res.isAnon() ){
		output += res.getId().toString();
	    }
	    else{
		output += res.getURI();
	    }
	}
	//System.out.println( output );

	return output;
    }

}
