/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.functionality;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.vocabulary.*;

/**
   Implementation of {@link RDFProperty} using Jena.
   <p>
   This requires the <a href="http://jena.sourceforge.net/">Jena API</a>
   to be installed on your system and listed in java's CLASSPATH.
   
   @see JenaPredicate
   @see JenaObject
   @author James Nash
*/
public class JenaProperty implements RDFProperty{

    // this property's data from the Jena RDF model 
    private Statement statement;

    // begin ugly hack!
    private JenaObject tmp2;
    private String tmp;

    /**
       Creates a new JenaProperty from a Jena API Statement.

       @param statement The Jena Statement to use
    */
    public JenaProperty(Statement statement){

	this.statement = statement;

	try{
	    this.tmp = statement.getString();
	}
	catch(Exception e){
	    this.tmp = "";
	}
	tmp2 = null;

    }


    /**
       Sets the object of this property.

       @param obj The object (aka value) to use
    */
    public void setObject( RDFObject obj ){
	
	// we expect Jena implementations here
	JenaObject jobj = (JenaObject) obj;

	if( jobj.getObjType() == RDFObject.LITERAL ){
	    // it's a literal
	    setObject( jobj.getLiteral() );
	}
	else{
	    // it's an RDFNode of some kind
	    statement.changeObject( jobj.getJenaNode() );

	    // ugly hack
	    tmp2 = jobj;

	}

    }

    /**
       Sets the object of this property to a literal value.
       Shorthand method to save making a literal <code>RDFObject</code> first.

       @param obj the literal value of the object of this statement
    */
    public void setObject( String obj ){

	// really do it
	statement.changeObject( obj );
	
	// fake it to fix weird GUI bug
	tmp = obj;
   }


    /**
       Query this property's predicate.
       
       @return the predicate
    */
    public RDFPredicate getPredicate(){
	JenaPredicate prop = new JenaPredicate( statement.getPredicate() );
	return prop;

    }

    /**
       Query this property's object.

       @return the object
    */
    public RDFObject getObject(){

	RDFObject output = null;
	      
	// ugly hack
	if ( tmp2 != null ){
	    return tmp2;
	}
	
	try{
	    // assume the object is a literal
	    String obj = statement.getString();

	    output = new JenaObject( tmp );
	}
	catch( Exception f ){
	    // if not it must be an RDFNode of some kind.
	    RDFNode obj = null;
	    
	    // in any case we sould be able to get a resource
	    Resource tmpR = statement.getResource();

	    // coz then we can check if it's a container...
	    if ( tmpR.hasProperty( RDF.type, RDF.Bag ) ){
		// it's a bag!
		//System.out.println("Attempting to get bag!");
		obj = statement.getBag();
	    }
	    else if ( tmpR.hasProperty( RDF.type, RDF.Seq ) ){
		// it's a seq!
		//System.out.println("Attempting to get sequence!");
		obj = statement.getSeq();
	    }
	    else if ( tmpR.hasProperty( RDF.type, RDF.Alt ) ){
		// it's a alt!
		//System.out.println("Attempting to get alternatives!");
		obj = statement.getAlt();
	    }	
	    else{
		// nope definitely normal resource
		obj = tmpR;

		//obj = statement.getModel().createResource(tmp2);
	    }

	    output = new JenaObject( obj );
	}

	return output;
    }

    /**
       Deletes this property.
    */
    public void delete(){
	statement.remove();
    }


}
