/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.functionality;

import java.io.*;
/**
   Interface for accessing, modifying and creating data in an RDF/XML file.
   Anything that can be uniquely identified by a URI is considered a <em>resource</em>.
   An RDF file may contain any number of resources, each with one or more
   <em>properties</em> describing them. When the value of a resource property
   itself has properties but no URI to identify it it is known as an
   <em>anonymous resource</em>.
   <p>
   This interface provides methods for accessing and adding resources in an RDF/XML
   file, creating predicates and objects to form properties about a resource and
   dealing with the actual file input/output.

   @see RDFResource
   @see RDFProperty
   @see RDFPredicate
   @see RDFObject

   @author James Nash
*/
public interface RDFFile{


    /**
       Creates blank RDF data in memory.
       If previous RDF data is present it is cleared.
    */
    void newFile();

    /**
       Reads in RDF data from a given RDF/XML file.

       @param file the file to load
       @throws RdfIOException if the file cannot be loaded or contains invalid RDF/XML
     */
    void loadFile( File file ) throws RdfIOException;

    /**
       Saves the current RDF data to the current file.

       @throws RdfIOException if there is a write error
    */
    void saveFile() throws RdfIOException;

    /**
       Saves the loaded RDF data to a file different from the current one.

       @param file the file to write to
       @throws RdfIOException if there is a write error
    */
    void saveFile( File file ) throws RdfIOException;

    /**
       Returns the name of the file from which the currently loaded RDF data originates. This is usually the name of the file which was loaded unless a new file has been created since then or the data was written to a different file.

       @return the current file name (null if there is no filename).
       @see #newFile()
       @see #loadFile(File)
       @see #saveFile(File)
     */
    String getFileName();


    /** 
	Retrieves all resources from the currently loaded data that have
	properties describing them.
	This includes anonymous resources.
	
	@return array of all the resources that are currently loaded
    */
    RDFResource[] getResources();



    /**
       Creates a new Resource.

       @param uri the URI of what the resource is describing
       @return a new RDFResource
    */
    RDFResource createResource(String uri);

    /**
       Creates a new anonymous Resource.
      
       @return a new anonymous RDFResource
    */
    RDFResource createResource();



    /**
       Creates a new Bag container.

       @param uri the URI of what the bag is describing
       @return a new RDFBag
    */
    RDFBag createBag(String uri);

    /**
       Creates a new anonymous bag container.
      
       @return a new anonymous RDFBag
    */
    RDFBag createBag();


    /**
       Creates a new Seq container.

       @param uri the URI of what the seq is describing
       @return a new RDFSeq
    */
    RDFSeq createSeq(String uri);

    /**
       Creates a new anonymous seq container.
      
       @return a new anonymous RDFSeq
    */
    RDFSeq createSeq();

    /**
       Creates a new Alt container.

       @param uri the URI of what the alt is describing
       @return a new RDFAlt
    */
    RDFAlt createAlt(String uri);

    /**
       Creates a new anonymous alt container.
      
       @return a new anonymous RDFAlt
    */
    RDFAlt createAlt();



    /**
       Creates a new <code>RDFPredicate</code> for when building a statement.

       @param type the Predicate's type (usually a URI)
       @return a new RDFPredicate
    */
    RDFPredicate createPredicate(String type);

    /**
       Create a literal <code>RDFObject</code> for when building a statement.

       @param val the object's literal value
       @return a new RDFObject
    */
    RDFObject createObject(String val);

    /**
       Creates a resource <code>RDFObject</code> for when building a statement.

       @param res the RDFResource to use
       @return a new RDFObject
    */
    RDFObject createObject(RDFResource res);


    /**
       Create an <code>RDFObject</code> that is a container.

       @param con the RDFContainer to use
       @return a new RDFObject
    */
    RDFObject createObject(RDFContainer con);


    /**
       Returns a list of known resource types.
       The list is an array of 2 element arrays.
       Each sub-array has a discriptive name as
       its first element and the full URI of the
       tpye as its second element.

       @return list of known resource type names and URIs
     */
    String[][] getResTypes();


    /**
       Returns a list of known property types.

       @return list of known predicates
    */
    RDFPredicate[] getPropTypes();

}
