/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.functionality;

/**
   An exception during input or output of RDF/XML data.
   This exception is designed to enable implementations of
   {@link RDFFile} to report common RDF/XML file read and
   write errors to Ganesha's GUI so that they may be handled in
   a sensible way. (For example: alerting the user)

   @author James Nash
*/
public class RdfIOException extends Exception{

    /**
       The type of error this Exception represents.
    */
    private int error;

    // possible input error types
    /**
       No valid RDF data was found in the file.
    */
    public static final int INVALID_RDF = 1;

    /**
       The requested file could not be found on the
       filesystem.
    */
    public static final int FILE_NOT_FOUND = 2;

    /**
       Other read errors.
       (no permissions, file locked etc..)
    */
    public static final int OTHER_READ_ERROR = 3;
    
    /**
       The file to be saved already exists.
    */
    public static final int FILE_EXISTS = 4;


    // possible output error types
    /**
       User does not have permission to write the file.
    */
    public static final int NO_WRITE_PERM = -1;
    
    
    
    /**
       Constructs a new RdfIOException for a given error.

       @param error the type of error
       @see #INVALID_RDF
       @see #FILE_NOT_FOUND
       @see #OTHER_READ_ERROR
       @see #FILE_EXISTS
       @see #NO_WRITE_PERM
    */    
    public RdfIOException( int error ){    
        this.error = error;    
    }


    /**
       Get a brief description of the kind of error.

       @return short error message
    */
    public String getMessage(){
        String cause = "An error (code: "+error+") has occurred while ";
        if ( error > 0 ){
            cause += "reading";
        }
        else{
            cause += "writing";
        }
        cause += " an RDF file.";
            
        return cause;
    }
    
    
    /**
       Get the type of error.
       The returned integer can be compared with the
       constant values defined above to determine what
       kind of error occured.

       @return the error code
       @see #INVALID_RDF
       @see #FILE_NOT_FOUND
       @see #OTHER_READ_ERROR
       @see #NO_WRITE_PERM
    */
    public int getError(){
        return error;
    }
    
}
