/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.gui;

import java.awt.event.*;

/**
   ActionListener for various menu and toolbar items.
   This ActionListener can be attached to menu or toolbar
   items that need to call methods in the {@link GUI} class.

   @author James Nash
*/
class GUIEvent implements ActionListener{

    /**
       What {@link GUI} method should be called?
    */
    private int type;

    /**
       Reference to the main app window
    */
    private GUI window;

    // Define constants for the possible results of an action

    /**
       Instruct Ganesha to shut down.
       
       @see GUI#shutdown()
    */
    public static final int SHUTDOWN = 0;

    /**
       Instruct Ganesha to open a file.

       @see GUI#openFile()
    */
    public static final int OPEN_FILE = 1;

    /**
       Instruct Ganesha to close a file.

       @see GUI#closeFile()
    */
    public static final int CLOSE_FILE = 2;

    /**
       Instruct Ganesha to save a file.

       @see GUI#saveFile()
    */
    public static final int SAVE_FILE = 3;
 
   /**
       Instruct Ganesha to save a file under a new name.

       @see GUI#saveFileAs()
    */
    public static final int SAVE_FILE_AS = 4;

    /**
       Instruct Ganesha to create a new file.

       @see GUI#newFile()
    */
    public static final int NEW_FILE = 5;

    /**
       Emulate the modification of a loaded file.
       This is intended for debugging purposes only.
    */
    public static final int MOD_FILE = -1;

    /**
       Instruct Ganesha to show the "about" screen.

       @see GUI#newFile()
    */
    public static final int ABOUT_SCREEN = 6;

    /**
       Show the HTML linking dialog.

       @see GUI#newFile()
    */
    public static final int HTML_LINK = 7;

    /**
       Create a new GUIEvent ActionListener.
       
       @param window reference to the main window object containing the methods to call.
       @param type determines which {@link GUI} method gets called when an ActionEvent is received
       @see #SHUTDOWN
       @see #OPEN_FILE
       @see #CLOSE_FILE
       @see #SAVE_FILE
       @see #SAVE_FILE_AS
       @see #NEW_FILE
    */
    public GUIEvent( GUI window, int type ){

	this.window = window;
	this.type = type;

    }


    /**
       Performs the action as determined by the constructor.

       @param e ignored
       @see #GUIEvent(GUI, int)
    */
    public void actionPerformed(ActionEvent e){
	

	switch (type){

	case SHUTDOWN:
	    window.shutdown();
	    break;

	case OPEN_FILE:
	    window.openFile();
	    break;
		
	case CLOSE_FILE:
	    window.closeFile();
	    break;
	    
	case NEW_FILE:
	    window.newFile();
	    break;
	
	case MOD_FILE:
	    window.setEdited( true );
	    break;

	case SAVE_FILE:
	    window.saveFile();
	    break;

	case SAVE_FILE_AS:
	    window.saveFileAs();
	    break;

	case ABOUT_SCREEN:
	    SplashScreen about = new SplashScreen( window );
	    about.show();
	    break;

	case HTML_LINK:
	    HtmlLinker link = new HtmlLinker( window );
	    break;

	}


    }


}
