/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.gui;

import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import com.twiddles.ganesha.gui.gfx.*;
import com.twiddles.ganesha.*;

/**
   Dialog to let the user insert a link to the current
   RDFFile into an existing (X)HTML file.

   @author James Nash
*/
class HtmlLinker extends JDialog{

    // window dimensions
    private final int WIDTH = 400;
    private final int HEIGHT = 270;

    /**
       Constructs and displays an HTML linking dialog.
       This allows the user to select an HTML file from
       which they wish to link to their current RDF/XML file.
       Ganesha will then automatically insert a 'link' tag pointing
       to the RDF file into the header of HTML page.

       @param owner the frame this window should be centred on
    */
    public HtmlLinker(GUI owner){
	// create a modal window
	super( owner, "Insert link into webpage", true);

	setResizable( false );
	setSize(WIDTH, HEIGHT);

	// center it above the parent window...
	int xOffset = (( owner.getWidth() - WIDTH) / 2);	    
	int yOffset = (( owner.getHeight() - HEIGHT) / 2);
	setLocation( new Point(( owner.getX() + xOffset ), ( owner.getY() + yOffset )) );


	// figure out our file...
	final String rdfFile = owner.getRDF().getFileName();

	// make buttons...
	JButton cancelButton = new JButton("Cancel");
	// make cancel button close window!
	cancelButton.addActionListener( new ActionListener() {

		public void actionPerformed(ActionEvent e){
		    // close the dialog
		    HtmlLinker.this.dispose();
		}

	    });

	JButton okButton = new JButton("Insert Link");
	
	Box buttons = Box.createHorizontalBox();
	buttons.add( Box.createHorizontalGlue() );
	buttons.add( okButton );
	buttons.add( Box.createHorizontalStrut( 5 ) );
	buttons.add( cancelButton );


	// contents of window...
	GridBagLayout layout = new GridBagLayout();
	JPanel panel = new JPanel(layout);
	panel.setBorder( BorderFactory.createCompoundBorder(
							    BorderFactory.createEmptyBorder(2,5,2,5),
							    BorderFactory.createCompoundBorder(
											       BorderFactory.createTitledBorder("Insert link into webpage"),
											       BorderFactory.createEmptyBorder(5,5,5,5)
											       )
							    )
			 );	
	GridBagConstraints gbc = new GridBagConstraints();
	gbc.gridx = 0;
	gbc.gridy = 0;
	gbc.gridwidth = 3;
	gbc.gridheight = 1;
	gbc.fill = GridBagConstraints.BOTH;

	if ( rdfFile == null ){
	    // no file yet...
	    
	    panel.add( new JLabel("You need to save your current work to a file"), gbc);
	    gbc.gridy++;
	    panel.add( new JLabel("before you can use this feature! Sorry!"), gbc );

	    okButton.setEnabled( false );
	}
	else{
	    // go ahead...
	    panel.add( new JLabel("Please choose an XHTML file into which you want to"), gbc);
	    gbc.gridy++;
	    panel.add( new JLabel("insert a link to \""+rdfFile+"\"."), gbc );
	    gbc.gridy++;
	    panel.add( new JLabel(" "), gbc );
	    gbc.gridy++;

	    gbc.gridwidth = 2;
	    final JTextField entry = new JTextField(20);
	    panel.add( entry, gbc);
	    
	    gbc.gridx = 2;
	    gbc.gridwidth = 1;
	    JButton browse = new JButton("Browse...");
	    // make the browse button pop up a file chooser
	    browse.addActionListener( new ActionListener() {

		    public void actionPerformed(ActionEvent e){
			JFileChooser fileDialogue = new JFileChooser( Settings.getLastSaveAsDir() );
			// only show rdf files..
			HTMLFileFilter filter = new HTMLFileFilter();
			fileDialogue.addChoosableFileFilter( filter );
			fileDialogue.setFileFilter( filter );
			
			int choice = fileDialogue.showOpenDialog(HtmlLinker.this);
			
			// did the user pick something or cancel?
			if ( choice == JFileChooser.APPROVE_OPTION ){
			    File myFile = fileDialogue.getSelectedFile();
			    if ( myFile.canWrite() ){

				entry.setText( myFile.getPath() );

			    }
			    else{

				JOptionPane.showMessageDialog(HtmlLinker.this, "You do not appear to have write permission\nfor the file \""+myFile.getName()+"\n\"Please choose a different file.", "No write permission", JOptionPane.WARNING_MESSAGE );

			    }
			}
			// if user cancelled do nothing
		    }
		});

	    // make the 'insert' button do the magic!
	    okButton.addActionListener( new ActionListener() {

		    public void actionPerformed(ActionEvent e){

			String fName = entry.getText().trim();
			if ( fName.length() != 0 ){

			    File file = new File( fName );

			    String contents;

			    try {
				FileInputStream input = new FileInputStream(file); // open a file input stream
				byte by[] = new byte[(int)file.length()];
				input.read(by);
				input.close(); // close the input stream
				contents = new String(by); // assign variable to the string from input stream
				
				
				// insert link...
				int headStart = contents.toLowerCase().indexOf("<head");
				int headEnd = contents.indexOf( '>', headStart);
				
				String link = "\n<link rel=\"meta\" href=\""+rdfFile+"\" type=\"application/rdf+xml\" />";
				headEnd++;
				
				String newContents = contents.substring(0, headEnd)+link+contents.substring(headEnd, contents.length());
				
				// write back to file...
				try{
				    PrintWriter save = new PrintWriter(new FileWriter(file));
				    save.print(newContents);
				    save.close();
				    
				    // we're done! close the editor!
				    HtmlLinker.this.dispose();
				}
				catch (IOException io2 )  {
				    JOptionPane.showMessageDialog(HtmlLinker.this, "There was an error saving the updated XHTML file:\n"+io2.toString(), "Error saving file.", JOptionPane.ERROR_MESSAGE );			    
				}
				
			    }
			    catch(java.io.IOException io) {
				JOptionPane.showMessageDialog(HtmlLinker.this, "There was an error opening the XHTML file:\n"+io.toString(), "Error opening file.", JOptionPane.ERROR_MESSAGE );
			    }
			}
			else{
			    // no file name specified...
			    JOptionPane.showMessageDialog(HtmlLinker.this, "You must specify an XHTML file.\nPlease type a filename into the box or\nclick the \"Browse\" button.", "No file specified.", JOptionPane.WARNING_MESSAGE );
			}
		    }
		});

	    panel.add( browse, gbc );

	}
	// add to window
	getContentPane().add( panel, BorderLayout.CENTER );
	getContentPane().add( buttons, BorderLayout.SOUTH );

	show();
    }



    /**
       HTML file filter for the file choosing dialogs.
       Filters out any non-html files based on their file extensions.
       
       @author James Nash
    */
    class HTMLFileFilter extends javax.swing.filechooser.FileFilter{

	/**
	   List of permitted HTML filename extensions.
	   The extensions are provided in lowercase and without
	   the preceding dot.
	*/
	private final String[] rdfExtensions = {"html", "htm", "xhtml", "xhtm"};
	

	/**
	   Extract a file's extension.        
	   The code for this method is based on an example from the
	   "Java(tm) tutorial, How to Use File Choosers", available at:
	   <a href="http://java.sun.com/docs/books/tutorial/uiswing/components/filechooser.html#filters">http://java.sun.com/docs/books/tutorial/uiswing/components/filechooser.html#filters</a>
	   <p>
	   Everything after the first dot in the filename is considered the extension.
	   
	   @param file the file whose filename extension is to be extracted
	   @return the file's extension converted to lowercase; null if there is no extension
	*/
	private String getExtension(File file){
	    String ext = null;
	    String s = file.getName();
	    int i = s.lastIndexOf('.');
	    
	    if (i > 0 &&  i < s.length() - 1) {
		ext = s.substring(i+1).toLowerCase();
	    }
	    return ext;
	    
	}


	/**
	   Filters out files without valid HTML extensions.
	   
	   @param file a file to be filtered
	   @return true if the file has an HTML extension or is a directory; false otherwise
	*/        
	public boolean accept( File file ){
	    if ( file.isDirectory() ){
		// need to allow dirs, so user can navigate filesystem
		return true;
	    }
	    
	    // check extension
	    String ext = getExtension(file);
	    if ( ext != null ){
		
		boolean result = false;
		
		for (int i=0; i<rdfExtensions.length; i++){
		    if ( ext.equals( rdfExtensions[i] ) ){
			// found a match!
			result = true;
			break;
		    }
		}
		return result;
	    }
	    // if there is no extension...
	    return false;    
	}

    
	/**
	   Returns description of this filter.
	   This short description is intended to be displayed in a
	   filechoosing widget's list of filetype filters.
	   
	   @return short descriptive text
	*/
	public String getDescription(){
	    return "HTML files (*.html, *.html, *.xhtml)";
	}
    }




}
