/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.gui;

import java.awt.*;
import java.util.Vector;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import com.twiddles.ganesha.functionality.*;
import com.twiddles.ganesha.gui.gfx.*;

/**
   The list of RDF resources for the main app window.
   This list appears on the right-hand side of the main app window
   and contains a list of all resources in the currently loaded RDF/XML
   file (it is blank if no file is currently loaded). It allows users
   to select resource and modify or remove them. There is also a button to
   add new resources.

   @author James Nash
*/
class ResourceList extends JPanel{

    // buttons under the resource list...
    /**
       Button to add new resources.
    */
    private JButton addButton;
    
    /**
       Button to remove selected resources.
    */
    private JButton removeButton;

    /**
       Button to edit selected resource.
    */
    private JButton editButton;
    
    /**
       Container for the list of resources.
    */
    private JScrollPane pane;
    
    /**
       Reference to the main app window.
    */
    private GUI window;

    /**
       Reference to the internal desktop of the main app window.
       This is used to communicate with the desktop in order
       to open / close resource editing windows.
    */
    private GDesktop desktop;

    /**
       The table that holds the data of the resource list
    */
    private JTable jt;


    /**
        Maintains list of open container windows
    */
    private Vector conWins;

 

    /**
        Constructs a fully functional resource list.

	@param window Reference to the main app window that will contain this list
	@param desktop Reference to the internal desktop where resource editing windows will appear
    */
    public ResourceList(GUI window, GDesktop desktop){

	this.window = window;
	this.desktop = desktop;
	this.jt = null;
        this.conWins = null;

        // set the layout
        setLayout(new BorderLayout());


        // prepare buttons
        addButton = new JButton( GUIIcons.getIcon( GUIIcons.ADD_RESOURCE_ICON ) );
	addButton.setToolTipText("Add a new resource to describe...");
	addButton.addActionListener( new ResourceListEvent( ResourceListEvent.ADD_RES, this, window ) );

        removeButton = new JButton( GUIIcons.getIcon( GUIIcons.REMOVE_RESOURCE_ICON ) );
	removeButton.setToolTipText("Remove the selected resource(s)");
	removeButton.addActionListener( new ResourceListEvent( ResourceListEvent.DEL_RES, this, window ) );

        editButton = new JButton( GUIIcons.getIcon( GUIIcons.EDIT_RESOURCE_ICON ) );
	editButton.setToolTipText("Edit the selected resource(s)"); 
	editButton.addActionListener( new ResourceListEvent( ResourceListEvent.EDIT_RES, this, window ) );


	// make buttons more square
	//int margin = removeButton.getMargin().top;
	//removeButton.setMargin( new Insets(margin, margin, margin, margin) );
	//editButton.setMargin( new Insets(margin, margin, margin, margin) );
	//addButton.setMargin( new Insets(margin, margin, margin, margin) );


	// just show blank pane at first
	pane = new JScrollPane();
        showBlank();

        // make the buttons that go under the list
        Box buttons = Box.createHorizontalBox();
        // space on the left...
        buttons.add(Box.createHorizontalGlue());
	// ...the buttons...
        buttons.add(addButton);
        buttons.add(removeButton);
        buttons.add(editButton);
	// ...and space on the right so they get centred
        buttons.add(Box.createHorizontalGlue());

	buttons.setBorder( BorderFactory.createEmptyBorder(5,5,5,5) );


        this.add(pane, BorderLayout.CENTER);
        this.add(buttons, BorderLayout.SOUTH);

    }

    
    /**
       Closes all open resource editing windows.
    */
    private void closeWindows(){
        // close container windows...
        if ( conWins != null ){
            ContainerWindow tmp;
                        
            // check if a window for the given container is already open
            for ( int i=0; i < conWins.size(); i++){            
                tmp = (ContainerWindow) conWins.elementAt( i );

                tmp.dispose();
                
            }
            
            // get rid of conWins
            conWins = null;
        }
    
        // close resource windows...
	// ...unless there is no table (in which case there can be no windows)
	if ( jt != null ){
	    for( int i=0; i< jt.getRowCount() ; i++){
	    
		ResourceListEntry tmp = (ResourceListEntry) jt.getValueAt(i,1);
		tmp.closeWindow();
		
	    }
	}
    }

    /**
       Refreshes open resource and container windows.
    */
    public void refreshWindows(){
	
	JInternalFrame tmp;
	JInternalFrame[] windows = desktop.getAllFrames();
	
	for(int i=0; i< windows.length; i++){
        
	    tmp = windows[i];
            
            if (tmp instanceof ResourceWindow){
                ResourceWindow tmp2 = (ResourceWindow) tmp;
                tmp2.refresh();
                continue;
            }
            if ( tmp instanceof ContainerWindow){
                ContainerWindow tmp2 = (ContainerWindow) tmp;
                tmp2.refresh();
                continue;
            }
	}
    }

    /**
       Makes this list blank.
       Clears the resource list and disables the buttons when no file is loaded
    */
    public void showBlank(){
	
	// close any outstanding windows
	closeWindows();

	// no table
	jt = null;

	// make blank label
	JLabel nothing = new JLabel("No file is currently loaded", SwingConstants.CENTER);
	pane.setViewportView(nothing);

	// disable buttons
	addButton.setEnabled( false );
	removeButton.setEnabled( false );
	editButton.setEnabled( false );
    }



    /**
       Table model for the resource list.
       
       @author James Nash
    */
    class ResourceTableModel extends AbstractTableModel {
	 
	/**
	   Column names.
	*/
	private String[] cols = {"","About",""};


	/**
	   The table's data
	*/
	private Vector data;


	/**
	   Constructs a table model for the resource list.
	   
	   @param rdfFile the RDF file from which to get the data
	   @param desktop reference to desktop where resource editing windows appear
	*/
	public ResourceTableModel( RDFFile rdfFile, GDesktop desktop ){
				  
	    //listenerList = new EventListenerList();

	    // get resources that are described in this file
	    RDFResource[] tmp = rdfFile.getResources();
   
	    // and load them into the table's data
	    data = new Vector();
	    for( int i=0; i< tmp.length; i++){		
		data.add( new ResourceListEntry( tmp[i], desktop, ResourceList.this ) );
	    }
	}



	/**
	   How many columns our table has.
	   
	   @return amount of columns in table
	*/
	public int getColumnCount() { return 3; }

	/**
	   Amount of rows in table.
	   
	   @return amount of rows in table
	*/
	public int getRowCount() { return data.size(); }

	/**
	   Retrieve a column's name.

	   @param col the column whose name you want
	   @return column's name
	*/
	public String getColumnName( int col ){
	    return cols[ col ];
	}

	/**
	   Get value of a certain cell.

	   @param row row coordinate
	   @param col col coordinate
	   @return cell's value
	*/
	public Object getValueAt(int row, int col){

	    Object value = null;

	    ResourceListEntry res = (ResourceListEntry) data.elementAt( row );

	    // first column is resource icon
	    if ( col == 0 ){
		value = res.getIcon();
	    }
	    // second col is resource list entry
	    else if ( col == 1 ){
		value = res;
	    }
	    // third col is window status icon
	    else if ( col == 2 ){
		value = res.getStatIcon();
	    }

	    return value;

	}

	/**
	   Get the type of class used in each column.

	   @param col the column for which to get the class
	   @return that column's class
	*/
	public Class getColumnClass(int col) {
	    return getValueAt(0, col).getClass();
	}
	
	/**
	   Determine if a given cell is editable.
	   In this case none of the cells are!

	   @param row ignored
	   @param col ignored
	   @return always false
	*/
	public boolean isCellEditable(int row, int col){
            
	    return false;
	}

	
	/**
	   Adds a new row to the table.

	   @param res the resource to add
	*/
	public void addRow(RDFResource res){

	    // add item to the data
	    data.add( new ResourceListEntry( res, desktop, ResourceList.this ) );

	    int index = data.size() - 1;

	    // notify any listeners that this has happened
	    fireTableRowsInserted(index, index);
	}

	/**
	   Deletes a given row from the table.

	   @param index the row to delete
	*/
	public void deleteRow(int index){
	    
	    // delete data
	    data.removeElementAt(index);
	    
	    // notify any listeners that this has happened
	    fireTableRowsDeleted(index, index);
	}

    }




    /**
       Renderer for resource icons that appear in the resource list.
       
       @author James Nash
    */
    class IconRenderer extends JLabel implements TableCellRenderer{

	/**
		Constructs a new icon renderer.
	*/
	public IconRenderer() {
	    setOpaque(true);
	}

	/**
		Returns a JLabel depicting the resource's icon.
		
		@param table The table whose cell is being rendered
    		@param icon The resource icon
    		@param isSelected whether or not this cell is selected
    		@param hasFocus ignored
    		@param row ignored
    		@param col ignored
	*/
	public Component getTableCellRendererComponent( JTable table, Object icon, boolean isSelected, boolean hasFocus, int row, int col){

	    if( isSelected ){
		setBackground( table.getSelectionBackground() );
		setForeground( table.getSelectionForeground() );
	    }
	    else{
		setBackground( table.getBackground() );
		setForeground( table.getForeground() );
	    }

	    Icon myIcon = (Icon) icon;
	    setIcon( myIcon );

	    return this;
	}
    }




    /**
       Renderer for resources that appear in the resource list.
       
       @author James Nash
    */
    class ResRenderer extends JLabel implements TableCellRenderer{

	/**
		Constructs a new resource renderer.
	*/
	public ResRenderer() {
	    setOpaque(true);
	    setFont( getFont().deriveFont( java.awt.Font.PLAIN ) );
	}

	/**
		Returns a JLabel depicting the resource's type and URI or ID.
		
		@param table The table whose cell is being rendered
    		@param res The resource
    		@param isSelected whether or not this cell is selected
    		@param hasFocus ignored
    		@param row ignored
    		@param col ignored
	*/
	public Component getTableCellRendererComponent( JTable table, Object res, boolean isSelected, boolean hasFocus, int row, int col){

	    if( isSelected ){
		setBackground( table.getSelectionBackground() );
		setForeground( table.getSelectionForeground() );
	    }
	    else{
		setBackground( table.getBackground() );
		setForeground( table.getForeground() );
	    }

	    ResourceListEntry myRes = (ResourceListEntry) res;
	    String subject = myRes.getResource().getURI();
	    String type = myRes.getResource().getTypeDescr();
	    
	    if ( myRes.getResource().isAnonymous() ){

		if ( type == null ){
		    setText("Anonymous resource (ID: "+subject+")");
		    setToolTipText("This is an anonymous resource with ID: "+subject);
		}
		else{
		    setText(type+" (Anonymous ID: "+subject+")");
		    setToolTipText("This is an anonymous resource of type "+type+" with ID: "+subject);
		}
	    }
	    else{

		if ( type == null ){
		    setText(subject);
		    setToolTipText("This is a resource describing "+subject);
		}
		else{
		    setText(type+" ("+subject+")");
		    setToolTipText("This is a resource of type "+type+" describing "+subject);
		}
	    }

	    return this;
	}
    }




    /**
       Renderer for window status icons that appear in the resource list.
       
       @author James Nash
    */
    class StatIconRenderer extends JLabel implements TableCellRenderer{

	/**
		Constructs a new window status icon renderer.
	*/
	public StatIconRenderer() {
	    setOpaque(true);
	}

	/**
		Returns a JLabel depicting the resource window status icon.
		
		@param table The table whose cell is being rendered
    		@param icon The window status icon
    		@param isSelected whether or not this cell is selected
    		@param hasFocus ignored
    		@param row ignored
    		@param col ignored
	*/
	public Component getTableCellRendererComponent( JTable table, Object icon, boolean isSelected, boolean hasFocus, int row, int col){

	    if( isSelected ){
		setBackground( table.getSelectionBackground() );
		setForeground( table.getSelectionForeground() );
	    }
	    else{
		setBackground( table.getBackground() );
		setForeground( table.getForeground() );
	    }

	    ImageIcon myIcon = (ImageIcon) icon;
	    setIcon( myIcon );
	    setToolTipText( myIcon.getDescription() );

	    return this;
	}
    }



    /**
       ActionListener to detect user selections in the resource list.

       @author James Nash
    */
    class ResourceSelectionListener implements ListSelectionListener{

	/**
		Enables or disables the 'remove resource' and 'edit resource' buttons when rows of the
		resource list are selected by the user. If one or more properties are seleced both buttons will be
		enabled otherwise neither button will be available.

		@param e The type of list selection event that is occurring
	*/
	public void valueChanged(ListSelectionEvent e){

	    // ignore adjustments
	    if (e.getValueIsAdjusting()){
		return;
	    }

	    ListSelectionModel l = (ListSelectionModel)e.getSource();
	    if (l.isSelectionEmpty()) {
		// if no rows are selected
		// the remove and edit buttons should be disabled
		removeButton.setEnabled( false );
		editButton.setEnabled( false );		

	    }
	    else{
		// something is selected
		//
		removeButton.setEnabled( true );
		editButton.setEnabled( true );

	    }
	}


    }




    /**
       Fills the list with the resources in the given file.
      
       @param rdfFile The RDF data from the file whose contents should be displayed
    */
    public void showFile(RDFFile rdfFile){

	// close any outstanding windows
	closeWindows();

	// reset starting position for new windows...
	desktop.resetLocation();


	// enable add button
	addButton.setEnabled( true );
        addButton.requestFocusInWindow();


        // create the table with our custom table model
	jt = new JTable( new ResourceTableModel( rdfFile, desktop ) );

	// set column width and renderer
	TableColumn col = jt.getColumnModel().getColumn(0);
	col.setMaxWidth(18);
	col.setMinWidth(18);
	col.setCellRenderer( new IconRenderer() );

	// set tooltips for middle column
	col = jt.getColumnModel().getColumn(1);
	col.setCellRenderer( new ResRenderer() );

	// set preferred column widths
	col = jt.getColumnModel().getColumn(2);
	col.setMaxWidth(18);
	col.setMinWidth(18);
	col.setCellRenderer( new StatIconRenderer() );

	// we only want to select rows, not columns
	jt.setColumnSelectionAllowed( false );

	// add a selection listener to enable or disable 
	// the edit and remove buttons
	jt.getSelectionModel().addListSelectionListener( new ResourceSelectionListener() );

	// add a mouselistener for editing rows when they are double clicked
	jt.addMouseListener(new MouseInputAdapter(){
		public void mouseClicked(java.awt.event.MouseEvent e){
		    // double click?
		    if (e.getClickCount() == 2){
			// which row (if any) was it?
			int[] entries = getSelectedEntries();
			if ( entries.length >= 1 ){
			    // only interested in one or more rows being double clicked
			    //System.out.println("row "+entries[0]+" double clicked" );

			    // pop up resource editing windows for selected resources!
			    for (int i=0; i<entries.length; i++){
				getEntry( entries[i] ).showWindow();
			    }
			    
			    // clear selection
			    clearSelect();

			}
			// else do fuck all! :P
		    }
		}
	    } );
	



	pane.setViewportView(jt);

    }

    /**
       Returns an array of all indices of entries in the resource list that
       are currently selected.

       @return array of selected indices
    */
    public int[] getSelectedEntries(){

	return jt.getSelectedRows();

    }

    /**
       Returns the entry value at a given index

       @param index indexs of the entry you want
       @return the entry at that position
    */
    public ResourceListEntry getEntry( int index){

	 ResourceListEntry output = (ResourceListEntry) jt.getValueAt(index, 1);
	 return output;
    }


    /**
       Repaints the ResorceList.
       This is for resource editing windows to request for the
       table to update itself to reflect any changes in window
       status icons.
    */
    public void refresh(){

	jt.repaint();

    }

    /**
       Clears any current selection of resources.
    */
    public void clearSelect(){
	
	jt.clearSelection();

    }

    /**
       Delete's the resource at the specified index.
       When deleting several resources this method
       should get called in reverse order of their
       indices in the table since deleting a row from
       the table will cause the indices of all following
       rows to derement by 1.

       @param index index of the resource to delete
    */
    public void delete(int index){

	ResourceTableModel model = (ResourceTableModel)jt.getModel();
	model.deleteRow( index );

    }


    /**
       Adds a new row to the table

       @param res the resource to add
    */
    public void add( RDFResource res ){

	ResourceTableModel model = (ResourceTableModel)jt.getModel();
	model.addRow( res );

    }


    /**
       Finds a given resource in the table.
       If the resource is found it's corresponding entry
       is returned. If nothing is found null is returned.

       @param res the resource to find
       @return the corresponding entry from the table
    */
    public ResourceListEntry findResource(RDFResource res){
	
	ResourceListEntry found = null;
	ResourceListEntry tmp;

	// pretty crude search
	// just go through the table looking for the resource!
	for (int i=0; i < jt.getRowCount(); i++){

	    tmp = (ResourceListEntry) jt.getValueAt(i, 1);

	    if( tmp.getResource().getURI() == res.getURI() ){
		//System.out.println("Resource found: "+res.getURI());
		found = tmp;
		break;
	    }

	}

	return found;
    }
    
    /**
        Displays a container window.

        @param con the container to show in the window
    */
    public void showContainerWindow( RDFContainer con ){
    
        if ( conWins == null ){
            // no others seem to be open so far...
            conWins = new Vector();
	}
	else{
	    ContainerWindow tmp;
	
	    // check if a window for the given container is already open
	    for ( int i=0; i < conWins.size(); i++){
            
		tmp = (ContainerWindow) conWins.elementAt( i );
		if ( tmp.getURI() == con.getURI() ){
		    // found a matching window!
		    // bring the window forward and give it focus!
		    // if window is minimised then undo that
		    if ( tmp.isIcon() ){
			try{
			    tmp.setIcon( false );
			}
			catch( Exception e ){
			    System.err.println("Could not de-iconify window!");
			}
		    }
		    tmp.moveToFront();
		    tmp.show();
		    
		    // we're done!
		    return;
		}                
		
	    }
	}
    
        // make a new window and show it
        ContainerWindow conWin = new ContainerWindow( con, this );

        conWins.add( conWin );
	
        // position the new window
        conWin.setLocation( desktop.getNextLocation() );
        desktop.add( conWin );

	conWin.show();
    }

    /**
        If a constainer window for the given container is open it gets closed.
        
        @param con the container to be displayed
    */
    public void closeContainerWindow( RDFContainer con ){
    
        ContainerWindow tmp;
        
        // find the window
        for ( int i=0; i < conWins.size(); i++){
        
            tmp = (ContainerWindow) conWins.elementAt( i );
            // is there a matching window?
            if ( tmp.getURI() == con.getURI() ){
            
                //System.out.println("Closing con window");
                tmp.dispose();
                conWins.removeElementAt( i );

		// make conWins null if there are no windows left
		if( conWins.size() == 0 ){
		    conWins = null;
		}

		break;
            }
        }

    }
    
    
    /**
        Promps the user to change a literal value.
        If the value is modified, the result is returned
        so that a resource or container window may display it 
        and the GUI is
        informed that the file has been modified.

        @param win the internal window the dialog should be centred to
        @param obj the RDF Object we are changing
        @return the new value if it was changed, else null
    */
    public String editLiteral(JInternalFrame win, RDFObject obj){
        
        String output = null;
	// trim excess whitespace from input
        String input = obj.getLiteral().trim().replaceAll("[ \t]+"," ");

        JTextArea textbox = new JTextArea( input, 6, 30);
        JScrollPane scroller = new JScrollPane( textbox, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        textbox.setLineWrap(true);
        textbox.setWrapStyleWord(true);
        textbox.setEditable(true);
        
        JLabel label = new JLabel("Property value:");
        
        Object[] contents = { label, scroller };
        
        int choice = JOptionPane.showConfirmDialog(win, contents, "Edit literal value", JOptionPane.OK_CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE);
        
        if ( choice != JOptionPane.CANCEL_OPTION ){
            // user clicked OK
            // get the user input
            output = textbox.getText();


	    //make sure it actually changed
	    if ( output.equals(input) ){
		// nope
		output = null;
	    }
	    else{
		// yep!
            
		// trim text
		output = output.trim().replaceAll("[ \t]+"," ");


		// notify the GUI
		window.setEdited(true);
	    }

            
	}

        return output;

    }
    
   

    /**
        Promps the user to change the URI of an undescribed resource.
        If the value is modified, the result is returned
        so that a resource or container window may display it 
        and the GUI is
        informed that the file has been modified.

        @param win the internal window the dialog should be centred to
        @param obj the RDF Object we are changing
        @return the new value if it was changed, else null
    */
    public RDFObject editResource(JInternalFrame win, RDFObject obj){
        
	RDFObject out = null;
        String output = null;
        String input = obj.getResource().getURI().trim();

        JTextField textbox = new JTextField( input, 30);            
        textbox.setEditable(true);
        
        JLabel label = new JLabel("Enter or modify the URI of this resource:");
        
        Object[] contents = { label, textbox };
        
        int choice = JOptionPane.showConfirmDialog(win, contents, "Edit resource URI", JOptionPane.OK_CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE);
        
        if ( choice != JOptionPane.CANCEL_OPTION ){
            // user clicked OK
            // get the user input
            output = textbox.getText();


	    //make sure it actually changed
	    if ( output.equals(input) ){
		// nope
		output = null;
	    }
	    else{
		// yep!
		
		// trim whitespace off the ends
		output = output.trim();
				

		// make new resource object
		out = window.getRDF().createObject( window.getRDF().createResource(output) );
		
		// notify the GUI
		window.setEdited(true);
		
	    }

            
	}

        return out;

    }

    
    /**
       Create's a RDF literal value.

       @param val the value of the literal
       @return an RDF object
    */
    public RDFObject createLiteral(String val){

	return window.getRDF().createObject( val );

    }


    /**
       Delete given rows from an ObjectTable.
       This will also instruct the GUI to consider
       the current file as having been edited.

       @param ot the table from which to delete the rows.
       @param rows the indices of the table rows to delete.
    */
    public void deleteObjectRows(ObjectTable ot, int[] rows){

	// need to delete them backwards...
	// sort the array
	java.util.Arrays.sort(rows);
	// go through it backwards
	for (int i=(rows.length-1); i>=0; i--){
	    ot.deleteRow( rows[i] );
	    
	    //System.out.println("Deleted row "+rows[i]);
	}

	window.setEdited( true );

    }


    /**
       Creates a combobox of resource types.
       This is intended fro use within the ResourceWindows.

       @param currentRes the current res type or null if the resource has no type
       @return a combo box containing all known resource types
    */
    public JComboBox getResTypes(RDFResource currentRes){

	String current = currentRes.getType();

	String[][] knownTypes = window.getRDF().getResTypes();


	// each element will be a 2 element array: descr + URI
	Vector input = new Vector();


	// no type
	input.add( "<Generic Resource>" );
	
	int selectedRow = -1;

	int i;
	for ( i=0; i< knownTypes.length; i++){

	    input.add( knownTypes[i] );

	    if( knownTypes[i][1].equals( current ) ){
		selectedRow = (i+1);
	    }

	}

	if ( selectedRow == -1){
	    if ( current != null ){
		// unknown type
		String[] tmp = {  currentRes.getTypeDescr() , currentRes.getType() };

		input.add( tmp );
		selectedRow = (i+1);
	    }
	    else{
		// default to < Generic resource >
		selectedRow = 0;
	    }
	}

	
	
	JComboBox output = new JComboBox( input );

	output.setRenderer( new ResTypeRenderer() );
	output.setSelectedIndex( selectedRow );


	return output;
    }


    /**
       Cell renderer for the resource type.

       @author James Nash
    */
    class ResTypeRenderer extends JLabel implements ListCellRenderer{

	/**
	   Constructs a new ResTypeRenderer
	*/
	public ResTypeRenderer(){
	    setOpaque( true );
	}

	/**
	   Render an item from the list

	   @param list the JList whose cell is being rendered
	   @param val the value of the cell
	   @param row the row of the cell	   
	   @param isSelected if the cell is currently selected
	   @param hasFocus if the cell has focus
	*/
	public Component getListCellRendererComponent(JList list, Object val, int row, boolean isSelected, boolean hasFocus){


	    if ( isSelected ){
		setBackground( list.getSelectionBackground() );
		setForeground( list.getSelectionForeground() );
	    }
	    else{
		setBackground( list.getBackground() );
		setForeground( list.getForeground() );
	    }


	    if ( val instanceof String ){
		// plain string
		setText( ((String) val) );
	    }
	    else{

		String[] tmp = (String[]) val;
		setText( tmp[0] );
		setToolTipText( tmp[1] );
	    }

	    // if text is to long shorten it
	    if ( getText().length() > 30 ){
		
		setText( getText().substring(0, 27) + ". . ." );

	    }

	    return this;
	}

    }


    /**
       Informs the GUI that the file has been edited.
    */
    public void didEdit(){
	window.setEdited( true );
    }


    /**
       Returns a reference to the GUI.

       @return reference to main app window
    */
    public GUI getGUI(){

	return window;

    }


    
}
