/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.gui;

import com.twiddles.ganesha.functionality.*;
import com.twiddles.ganesha.gui.gfx.*;
import javax.swing.*;

/**
   Container for the data that makes up an entry in the Resource list.
   This will hold, the resource, its icon and a reference to the
   editing window for that resource.

   @see ResourceList
   @see ResourceWindow

   @author James Nash
*/
class ResourceListEntry{

    /**
       The RDF Resource that this entry is about.
    */
    private RDFResource res;
    
    /**
       The editing window associated with the resource.
    */
    private ResourceWindow resWin;

    /**
       The associated resource icon.
    */
    private Icon icon;


    /**
       The window status icon.
       Shows if the window for this resource is currently open, iconified or closed.
    */
    private Icon statIcon;

    /**
       The internal desktop where the resource windows are to appear.
    */
    private GDesktop desktop;

    /**
       The resource list that contains this entry.
    */
    private ResourceList resList;


    /**
       Constructs a new list entry.
       A resource icon will automatically be generated. No resource
       window will be created at this point, accordingly the window
       status will be set to 'closed' for now.

       @param res the resource that this entry refers to
       @param desktop reference to the desktop where resource windows appear
    */
    public ResourceListEntry( RDFResource res, GDesktop desktop, ResourceList resList ){

	this.res = res;
	this.desktop = desktop;
	this.resList = resList;
	this.icon = ResIconFactory.getResIcon();
	this.resWin = null;

	this.statIcon = GUIIcons.getIcon( GUIIcons.WINDOW_CLOSED_ICON );
    }


    /**
       Show this resource's window.
       If there is currently no window for the resource a
       new one is created and displayed. If there is a
       window but it is iconified it is restored.
       Otherwise the window is simply brought forward
       and given focus.
    */
    public void showWindow(){

	if ( resWin == null ){

	    // there is no window yet!
	    resWin = new ResourceWindow( res, icon, resList );
	    resWin.addInternalFrameListener( new ResourceWindowListener(this, resList) );
	    // position the new window
	    resWin.setLocation( desktop.getNextLocation() );

	    desktop.add( resWin, JLayeredPane.DEFAULT_LAYER );
	    resWin.show();
	}
	else{
	    // if window is minimised then undo that
	    if ( resWin.isIcon() ){
		try{
		    resWin.setIcon( false );
		}
		catch( Exception e ){
		    System.err.println("Could not de-iconify window!");
		}
	    }
	    resWin.moveToFront();
	    // why does this bit not fucking work??? GRRRRRRRRRRR!!!
	    resWin.requestFocusInWindow();
	    
	}
    }


    /**
       Get this resource's icon.

       @return this resource's icon
    */
    public Icon getIcon(){

	return icon;
    }

    /**
       Get the window status icon for this resource.

       @return current image status icon
    */
    public Icon getStatIcon(){

	return statIcon;
    }


    /**
       Get the resource.
       
       @return reference to the resource
    */
    public RDFResource getResource(){

	return res;
    }


    /**
       Close this resource's window.
       If it is open that is, if not nothing happens.
    */
    public void closeWindow(){

	if( resWin != null ){
	    if( ! resWin.isClosed() ){
		resWin.dispose();
	    }
	    resWin = null;

	}

    }


    /**
       Set the window status icon.
    */
    public void setStatIcon( Icon statIcon ){
	this.statIcon = statIcon;
    }

}
