/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.gui;

import java.awt.event.*;
import javax.swing.JOptionPane;
import com.twiddles.ganesha.functionality.*;

/**
   ActionListener for buttons on the resource list.

   @see ResourceList
   @author James Nash
*/
class ResourceListEvent implements ActionListener{

    /**
       The type of action that should be performed by this
       listener.
    */
    private int type;


    /**
       Add a new resource.
    */
    public static final int ADD_RES = 1;

    /**
       Delete selected resource(s).

       @see ResourceList#getSelectedEntries()
       @see ResourceList#delete(int)
    */
    public static final int DEL_RES = 2;

    /**
       Edit selected resource(s).

       @see ResourceList#getSelectedEntries()
    */
    public static final int EDIT_RES = 3;

    /**
       Reference to the resource list containing the methods to perform
    */
    private ResourceList resList;

     /**
       Reference to the main GUI for updating it's states
    */
    private GUI window;


    /**
       Constructs an ActionListener that will perform
       the specified action when any event is received.

       @param type the type of action to perform
       @param resList reference to the resource list
       @param window reference to the main GUI window
       @see #ADD_RES
       @see #DEL_RES
       @see #EDIT_RES
    */
    public ResourceListEvent(int type, ResourceList resList, GUI window){
	this.type = type;
	this.resList = resList;
	this.window = window;
    }


    /**
       Performs the action given to the constructor.

       @param e ignored
       @see #ResourceListEvent(int, ResourceList, GUI)
    */
    public void actionPerformed(ActionEvent e){
	
	int[] entries = resList.getSelectedEntries();
	int z = 0;

	switch( type ){

	case ADD_RES:

	    // ask user for info to create new resource
	    AddWizard dialog = new AddWizard( window );
	    RDFResource myRes = dialog.addResource();

	    if ( myRes != null ){
		// a new resource was aded to the data!
		// add it to the table
		//System.out.println("Want to add resource "+myRes.getURI()+" to list!!" );

		resList.add( myRes );

		// adding a resource modifies the file!
		window.setEdited( true );
		
		// refresh open resource windows
		resList.refreshWindows();
	    }
	    // if it was null the user cancelled - do nothing!
	    break;

	case DEL_RES:
	    // need to to this in reverse order of indices
	    // since otherwise subsequent table entry indices
	    // change and we end up deleting the wrong ones!

	    // confirm that user wants to delete entries
	    String message = "the selected resource?";
	    if ( entries.length > 1 ){
		message = "the "+entries.length+" selected resources?";
	    }
	    int confirmation = JOptionPane.showConfirmDialog( resList, "Are you sure you want to delete\n"+message, "Confirm deleting resource(s)", JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);

	    if ( confirmation == JOptionPane.OK_OPTION ){
		// yes!

		// sort the array
		java.util.Arrays.sort(entries);

		// go through it backwards
		for (int i=(entries.length-1); i>=0; i--){
		    z = entries[i];
		    
		    // close this resource's window should it be open
		    resList.getEntry(z).closeWindow();
		    
		    // delete the resource from the RDFFile
		    resList.getEntry(z).getResource().delete();
		    
		    // remove table entry
		    resList.delete(z);
		    
		    //System.out.println("Deleted a resource! ("+z+")");
		}
		// deleting a resource modifies the file!
		window.setEdited( true );
		// refresh open resource windows
		resList.refreshWindows();
	    }
	    // else do nothing
	    break;


	case EDIT_RES:
	    // pop up resource editing windows for selected resources!
	    for (int i=0; i<entries.length; i++){
		resList.getEntry( entries[i] ).showWindow();
	    }
	    break;

	}

	// clear the selection of the list afterwards...
	resList.clearSelect();

    }


}
