/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.gui;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;
import java.util.Vector;
import com.twiddles.ganesha.functionality.*;
import com.twiddles.ganesha.gui.gfx.*;

/**
   A resource editing window.
   This is the main kind of window that may appear in Ganesha's internal
   desktop. It displays the properties and values of a given resource
   and allows the user to add, edit or delete them.

   @author James Nash
*/
class ResourceWindow extends JInternalFrame{


    /**
       The default width for new windows.
    */
    public static final int DEFAULT_WIDTH = 380;

    /**
       The default height for new windows.
    */
    public static final int DEFAULT_HEIGHT = 300;


    /**
       The table containing the property data
    */
    private JTable jt;

    // the resource contained in this window
    private RDFResource res;

    // The resource list that spawned this window
    private ResourceList resList;

    /**
       Button to add new properties.
    */
    private JButton addButton;
    
    /**
       Button to remove selected properties.
    */
    private JButton removeButton;

    /**
       Button to edit selected properties.
    */
    private JButton editButton;


    // list of available resource types...
    private JComboBox typeListComboBox;


    /**
       Constructs a new resource editing window.
       The resource's subject will appear as the window title.

       @param res the resource that should be displayed in this window
       @param icon the icon associated with the resource
       @param resList the resource list that spawned this window
    */
    public ResourceWindow( RDFResource res, Icon icon, ResourceList resList ){

	// make window with title that is resizable, closable, maximisable and iconifiable
	super( res.getURI(), true, true, true, true );
	// put the icon in the title bar
	setFrameIcon( icon );

	if ( res.getType() != null ){
	    setTitle( res.getTypeDescr()+" ("+res.getURI()+")" );
	}

	this.res = res;
	this.resList = resList;


	// set initial size
	// ...do this
	setSize(DEFAULT_WIDTH, DEFAULT_HEIGHT);


        // create Buttons
        addButton = new JButton( GUIIcons.getIcon( GUIIcons.ADD_PROPERTY_ICON ) );
        addButton.setToolTipText("Add a new property to this resource");

        removeButton = new JButton( GUIIcons.getIcon( GUIIcons.REMOVE_PROPERTY_ICON ) );
        removeButton.setToolTipText("Remove selected properties from this resource");

        editButton = new JButton( GUIIcons.getIcon( GUIIcons.EDIT_PROPERTY_ICON ) );
        editButton.setToolTipText("Edit the selected property");


	// make the add button work!
	addButton.addActionListener( new ActionListener(){

		// add a new property!
		public void actionPerformed( ActionEvent e ){

		    // ask user for info to create new resource
		    AddWizard dialog = new AddWizard( ResourceWindow.this.resList.getGUI() );
		    RDFProperty myProp = dialog.addProperty( ResourceWindow.this.res );
		    
		    if ( myProp != null ){
			// user made a property
			//System.out.println("Gonna add new property!");

			PropertyTableModel ptm = (PropertyTableModel) ResourceWindow.this.jt.getModel();
			ptm.addRow( myProp );

			// update gui
			ResourceWindow.this.resList.didEdit();
		    }
		    // else user cancelled

		}

	    });
    
    
        // make the buttons that go under the list
        Box buttons = Box.createHorizontalBox();
        // space on the left...
        buttons.add(Box.createHorizontalGlue());
	// ...the buttons...
        buttons.add(addButton);
        buttons.add(removeButton);
        buttons.add(editButton);
	// ...and space on the right so they get centred
        buttons.add(Box.createHorizontalGlue());

	buttons.setBorder( BorderFactory.createEmptyBorder(5,5,5,5) );

	// make type combobox...
	Box typeList = Box.createHorizontalBox();
	typeList.add(Box.createHorizontalGlue());
	typeList.add( new JLabel("Resource type: ") );
	// add the list
	this.typeListComboBox = resList.getResTypes( res );
	typeList.add( typeListComboBox );
	//typeList.add(Box.createHorizontalGlue());
	// button for making new types
	JButton newType = new JButton( "Custom..." );
	newType.setToolTipText("Define a custom resource type...");
	typeList.add( newType );
	typeList.add(Box.createHorizontalGlue());


	typeList.setBorder( BorderFactory.createEmptyBorder(5,5,5,5) );

	// add listener to the list	
	typeListComboBox.addActionListener( new ActionListener(){

		public void actionPerformed( ActionEvent e ){

		    JComboBox cBox = (JComboBox)e.getSource();

		    Object type = cBox.getSelectedItem();

		    if ( type instanceof String[] ){

			String[] myType = (String[]) type;

			// swap this resource's type
			ResourceWindow.this.res.setType( myType[1] );
			// update window title
			ResourceWindow.this.setTitle( ResourceWindow.this.res.getTypeDescr()+" ("+ResourceWindow.this.res.getURI()+")" );

		    }
		    else{
			// must be string
			String myType = (String) type;

			//System.out.println("Deleting type");

			// remove type!
			ResourceWindow.this.res.delType();
			ResourceWindow.this.setTitle( ResourceWindow.this.res.getURI() );

		    }

		    // refresh the reslist to show it
		    ResourceWindow.this.resList.refresh();
		    // notify GUI
		    ResourceWindow.this.resList.didEdit();
		    
		}

	    });

	// add listener to this button to allow users
	// to add new properties
        newType.addActionListener( new ActionListener(){

		public void actionPerformed( ActionEvent e ){

		    // ask user for type's URI
		    String output;

		    
		    JTextField textbox = new JTextField( "http://", 30);            
		    textbox.setEditable(true);
		    
		    JLabel label = new JLabel("Enter the URI of the resource type:");
		    
		    Object[] contents = { label, textbox };
		    
		    int choice = JOptionPane.showConfirmDialog(ResourceWindow.this, contents, "Enter resource type URI", JOptionPane.OK_CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE);
		    
		    if ( choice != JOptionPane.CANCEL_OPTION ){
			// user clicked OK
			// get the user input
			output = textbox.getText().trim();
			
			// swap this resource's type
			ResourceWindow.this.res.setType( output );

			// add type to list...
			String[] tmp = { ResourceWindow.this.res.getTypeDescr(), ResourceWindow.this.res.getType() };
			ResourceWindow.this.typeListComboBox.addItem( tmp );
			// and select it...
			ResourceWindow.this.typeListComboBox.setSelectedIndex( (ResourceWindow.this.typeListComboBox.getItemCount() - 1 ) );
			

			// update window title
			ResourceWindow.this.setTitle( ResourceWindow.this.res.getTypeDescr()+" ("+ResourceWindow.this.res.getURI()+")" );

			// refresh the reslist to show it
			ResourceWindow.this.resList.refresh();
			// notify GUI
			ResourceWindow.this.resList.didEdit();

		    }
		    // if the user cancelled do nothing!
		}

	    });
	



	// create table!
	jt = new JTable( new PropertyTableModel(res) );

	// set column width and renderer
	TableColumn col = jt.getColumnModel().getColumn(1);
	col.setCellRenderer( new ObjectRenderer( resList ) );
	//col.setCellEditor( new ObjectEditor( resList, this ) );
	col.setPreferredWidth(200);
	

	col = jt.getColumnModel().getColumn(0);
	col.setCellRenderer( new PredicateRenderer() );
	col.setPreferredWidth(70);

	// add a selection listener to enable or disable 
	// the edit and remove buttons
	jt.getSelectionModel().addListSelectionListener( new PropertySelectionListener() );
        

        
        // add a mouselistener for editing rows when they are double clicked
	jt.addMouseListener(new MouseInputAdapter(){
		public void mouseClicked(java.awt.event.MouseEvent e){
		    // double click?
		    if (e.getClickCount() == 2){
			// which row (if any) was it?
			int[] entries = getSelectedEntries();
			if ( entries.length == 1 ){
			    // only interested in one row being double clicked
                            
                            ObjectEditor.editObject( ResourceWindow.this.resList, ResourceWindow.this, jt, entries[0], 1 );
			                                
			    // clear selection
			    clearSelect();

                            refresh();
			}
			// else do fuck all! :P
		    }
		}
	    } );
	

        // disable / enable buttons correctly
        removeButton.setEnabled( false );
	editButton.setEnabled( false );	

	// make buttons more square
	//int margin = removeButton.getMargin().top;
	//removeButton.setMargin( new Insets(margin, margin, margin, margin) );
	//editButton.setMargin( new Insets(margin, margin, margin, margin) );
	//addButton.setMargin( new Insets(margin, margin, margin, margin) );

	// add listener to edit button
	editButton.addActionListener( new ActionListener(){

		public void actionPerformed( ActionEvent e ){

		    // which row (if any) was it?
		    int[] entries = getSelectedEntries();
		    if ( entries.length == 1 ){
			
			ObjectEditor.editObject( ResourceWindow.this.resList, ResourceWindow.this, jt, entries[0], 1 );
			
			// clear selection
			clearSelect();
			
			refresh();
		    }
		    // else do fuck all! :P
		    
		}

	    });


	// add listener to remove button
	removeButton.addActionListener( new ActionListener(){

		public void actionPerformed( ActionEvent e ){

		    int[] selection = getSelectedEntries();
		    // confirm that user wants to delete entries
		    String message = "the selected property?";
		    if ( selection.length > 1 ){
			message = "the "+selection.length+" selected properties?";
		    }
		    int confirmation = JOptionPane.showConfirmDialog( ResourceWindow.this, "Are you sure you want to delete\n"+message, "Confirm deleting properties", JOptionPane.OK_CANCEL_OPTION, JOptionPane.WARNING_MESSAGE);
		    if ( confirmation == JOptionPane.OK_OPTION ){
			// yes!

			ResourceWindow.this.resList.deleteObjectRows( ((ObjectTable) jt.getModel() ), selection );
		    }
		    else{
			// otherwise just clear selection
			clearSelect();
		    }
		}

	    });



	// create scroll pane to hold table
	JScrollPane spane = new JScrollPane();
	spane.setViewportView( jt );


	// put some stuff into the window
	JComponent pane = (JComponent) this.getContentPane();

	pane.add( typeList, BorderLayout.NORTH );
	pane.add( spane, BorderLayout.CENTER );
        pane.add( buttons, BorderLayout.SOUTH );

    }




    /**
       Table model for the properties of this resource
       
       @author James Nash
    */
    class PropertyTableModel extends AbstractTableModel implements ObjectTable{
	 
	/**
	   Column names.
	*/
	private String[] cols = {"Property", "Value"};


	/**
	   The table's data
	*/
	private Vector data;
	
	/**
	   Constructs a table model of the resource's properties.
	   
	   @param res the resource whose properties are listed in this table
	*/
	public PropertyTableModel( RDFResource res ){
				  
	    // get resources that are described in this file
	    RDFProperty[] tmp = res.getProperties();
   
	    // and load them into the table's data
	    data = new Vector();
	    for( int i=0; i< tmp.length; i++){		
		data.add( tmp[i] );
	    }
	}

	/**
	   How many columns our table has.
	   
	   @return amount of columns in table
	*/
	public int getColumnCount() { return 2; }

	/**
	   Amount of rows in table.
	   
	   @return amount of rows in table
	*/
	public int getRowCount() { return data.size(); }

	/**
	   Retrieve a column's name.

	   @param col the column whose name you want
	   @return column's name
	*/
	public String getColumnName( int col ){
	    return cols[ col ];
	}

        
        /**
            Get the RDF object in the given row.
            
            @param row the row containing the object to retrieve
        */
        public RDFObject getObjectAt(int row){
            RDFProperty stat = (RDFProperty) data.elementAt( row );
            return stat.getObject();
        }


	/**
	   Get the type of class used in each column.

	   @param col the column for which to get the class
	   @return that column's class
	*/
	public Class getColumnClass(int col) {
	    return getValueAt(0, col).getClass();
	}

	
	/**
	   Determine if a given cell is editable.
	   In this case none are!

	   @param row ignored
	   @param col ignored
	   @return always false
	*/
	public boolean isCellEditable(int row, int col) {

	    return false;
	}


	/**
	   Get value of a certain cell.

	   @param row row coordinate
	   @param col col coordinate
	   @return cell's value
	*/
	public Object getValueAt(int row, int col){

	    Object value = null;

	    RDFProperty stat = (RDFProperty) data.elementAt( row );

	    // first column is property
	    if ( col == 0 ){
		value = stat.getPredicate();
	    }
	    // second col is value
	    else if ( col == 1 ){
		value = stat.getObject();
		//System.out.println("Reading "+value.toString() );
	    }

	    return value;

	}



	/**
            Set's the value of cell in the table.
        
            @param obj the new RDF object to replace whatever is currently in that row
            @param row the row containing the property to be changed
        */
        public void setValueAt(Object obj, int row, int col){
        
	    // we can only change objects on the fly
	    // so the first column is ignored
	    if ( col == 1 ){
		// if it's the object column do stuff!
		
		RDFProperty prop = (RDFProperty) data.elementAt( row );

		if ( obj instanceof String ){
		    String robj = (String) obj;
		    // put the new object in		
		    prop.setObject( robj );
	
		    
		}
		else{
		    RDFObject robj = (RDFObject) obj;
		    // put the new object in
		    prop.setObject( robj );
		    
		    //System.out.println("Inserting "+robj.toString() );
		}

		

		// do table changed
		fireTableChanged( new TableModelEvent(this, row, row) );
		
	    }
        }
      

	/**
	   Deletes a given row from the table.

	   @param index the row to delete
	*/
	public void deleteRow(int index){

	    // delete the property from the RDF data
	    ((RDFProperty) data.elementAt(index)).delete();
	    
	    // delete data
	    data.removeElementAt(index);
	    
	    // notify any listeners that this has happened
	    fireTableRowsDeleted(index, index);
	}

	/**
	   Adds a new row to the table.

	   @param prop the resource to add
	*/
	public void addRow(RDFProperty prop){

	    // add item to the data
	    data.add( prop );

	    int index = data.size() - 1;

	    // notify any listeners that this has happened
	    fireTableRowsInserted(index, index);
	}



    }



    /**
       Renderer for the property (ie predicate) column.
       
       @author James Nash
    */
    class PredicateRenderer extends JLabel implements TableCellRenderer{


	/**
	   Constructs a new PredicateRenderer.
	*/
	public PredicateRenderer() {
	    setOpaque(true);
	    setFont( getFont().deriveFont( java.awt.Font.PLAIN ) );
	}

	/**
	   Returns a JLabel depicting the predicate's description.
	   The tooltip text is set as the predicate's type.
	   
	   @param table the table whose cell is being rendered
	   @param pred the predicate object
	   @param isSelected whether or not this cell is selected
	   @param hasFocus ignored
	   @param row ignored
	   @param col ignored
	   @return a rendered cell
	*/
	public Component getTableCellRendererComponent( JTable table, Object pred , boolean isSelected, boolean hasFocus, int row, int col){

	    if( isSelected ){
		setBackground( table.getSelectionBackground() );
		setForeground( table.getSelectionForeground() );
	    }
	    else{
		setBackground( table.getBackground() );
		setForeground( table.getForeground() );
	    }
	    
  	    RDFPredicate myPred = (RDFPredicate) pred;

	    this.setText( myPred.getDescr() );
	    this.setToolTipText( "Property type: "+myPred.getType() );
	    	   	    
	    return this;
	}
    }

    /**
       Refreshes the display of the table.
       Intended for use when resources have been added / deleted and
       contents of the resource window might change as a result.
    */
    public void refresh(){

	jt.repaint();

    }
    
    
    
     /**
       ActionListener to detect user selections in the property list.

       @author James Nash
    */
    class PropertySelectionListener implements ListSelectionListener{

	/**
		Enables or disables the 'remove property' and
		'edit property' buttons when rows of the property list
		are selected by the user. If one property is seleced both
		buttons will be enabled, if more than one property is selected
		only 'remove properties' will be enabled and if nothing is
		selected neither button will be available.

		@param e The type of list selection event that is occurring
	*/
	public void valueChanged(ListSelectionEvent e){

	    // ignore adjustments
	    if (e.getValueIsAdjusting()){
		return;
	    }

	    ListSelectionModel l = (ListSelectionModel)e.getSource();
	    if (l.isSelectionEmpty()) {
		// if no rows are selected
		// the remove and edit buttons should be disabled
		removeButton.setEnabled( false );
		editButton.setEnabled( false );		

	    }
	    else{
		// something is selected
		//
		removeButton.setEnabled( true );
                
                // can only edit one property at a time
                if ( jt.getSelectedRowCount() == 1 ){
                    editButton.setEnabled( true );
                }
                else{
                    editButton.setEnabled( false );
                }

	    }
	}
    }
    
    
    /**
       Clears any current selection of resources.
    */
    public void clearSelect(){
	
	jt.clearSelection();

    } 

    /**
       Returns an array of all indices of entries in the property list that
       are currently selected.

       @return array of selected indices
    */
    public int[] getSelectedEntries(){

	return jt.getSelectedRows();

    }
    



}
