/*
    Copyright 2004 James Nash
                                                                                                
    This file is part of Ganesha.
                                                                                                
    Ganesha is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
                                                                                                
    Ganesha is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
                                                                                                
    You should have received a copy of the GNU General Public License
    along with Ganesha (see the LICENSE file); if not, write to the Free
    Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
    MA  02111-1307  USA
                                                                                                
*/
package com.twiddles.ganesha.gui.gfx;

import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;

/**
    Producer of randomly coloured resource icons.
    
    @author James Nash
*/
public class ResIconFactory {

    /**
       The hue value of the last generated colour.
    */
    private static float lastHue = 0.346f;
    private static float lastSat = 1.0f;

    /** 
	Resource icon.
        Spherical, coloured icon used to associate a resource from the
	resource list with its associated editing window.

	@author James Nash
	@see com.twiddles.ganesha.gui.ResourceList
    */
    static class ResIcon implements Icon{
        
	/**
	   The colour of this icon
	*/
        private Color color;
        

	/**
	   Constructs a resource icon with a given colour.

	   @param color The colour the icon should have
	*/
        public ResIcon( Color color ){
            this.color = color;
        }
        
	/**
	   Query the icon's width.
	   
	   @return the icon's width in pixels
	*/
        public int getIconWidth(){
            return 16;
        }
        
	/**
	   Query the icon's height.

	   @return the icon's height in pixels
	*/
        public int getIconHeight(){
            return 16;
        }
        
        /**
            Draws a resouce icon.
	    Draws a round icon in the specified colour, adds a drop shadow and a highlight effect
	    to give it a spherical appearance.
        */
        public void paintIcon(Component c, Graphics g, int x, int y){
    
            Graphics2D g2d = (Graphics2D) g;
    
            // draw drop shadow
            g.setColor( new Color(0, 0, 0, 180) );
            g.fillOval( (x+1), (y+1), 15, 15 );
    
            // draw oval on top
            Color color2 = new Color(0,0,0);
            Ellipse2D el = new Ellipse2D.Double( x, y, 15, 15 );
            GradientPaint gp = new GradientPaint((x-8),(y-8), color2, (x+13), (y+13), color);
            g2d.setPaint(gp);
            g2d.fill(el);
            
            //g.setColor( color );
            //g.fillOval(x, y, 15, 15);
            
            // draw highlight!
            // ( or, if you prefer, slightly transparent, overlayed, concentric, white circles! )
            for ( int i=0; i<4; i++){
                g.setColor( new Color(255, 255, 255, (30+(i*22)) ) );
                g.fillOval( (x+i+2), (y+i+2), (7-(2*i)), (7-(2*i)) );
                
            }
                        
                                        
        }
    
    }
    
    
    /**
        Generates the next random colour.
	Actually the colours aren't really random: A small amount is added to the hue of the
	last produced colour to
	create a new one. Therefore subsequent calls to this method produce
	new colours that gradually cycle through the rainbow colours. The increment
	is chosen so that it does not wrap round the spectrum evenly and thus
	*lots* of different colours will be produced before there are any exact repeats.

	@return new pseudo-random colour
    */
    private static Color getNextColor(){
        Color output;

	float brightness = 1f;
        
        output = Color.getHSBColor( lastHue, lastSat, brightness );
        
	// increment the hue by some weird amount
        lastHue += 0.0893437f;
    
        // cycle round when hue reaches 1
        if ( lastHue >= 1f ){
            lastHue -= 1f;
	    lastSat -= 0.1685;

	    if ( lastSat <= 0.24f ){
		lastSat = 1f;
	    }
        }
    
        return output;
    }
    
      
    /**
        Get a new, randomly coloured resource icon.

	@return new resource icon
    */
    public static Icon getResIcon(){
    
        Icon icon = new ResIcon( getNextColor() );
        return icon;    
    }

    /**
       Get a grey version of the resource icon.
       These are intended for use on resources that do not have any described properties.

       @return new grey resource icon
    */
    public static Icon getGreyIcon(){

	Icon icon = new ResIcon( Color.getHSBColor( 1f, 0f, 0.7f ) );
	return icon;

    }

    
}
